import { distF, uuidv4 } from './util.js'
import Entity from './entity.js';

class Player extends Entity {
    constructor(you, isPlayer, game = false) {
        super(game);

        this.camera = { x: -this.pos.x, y: -this.pos.y };

        this.rot = 0;
        this.dir = 1;
        this.ticks = 0;

        this.health = 100;

        this.you = you || uuidv4();

        this.isPlayer = isPlayer;

        this.headCount = 0;

        this.type = 'Player';

        this.isMenu = false;

        this.r = 1;

        this.isYou = false;

        this.serverProps = [
            'type', 'camera', 'pos', 'vel', 'rot', 'dir', 'ticks', 'health', 'you', 'isPlayer', 'headCount', 'isMenu', 'r', 'isYou', 'username'
        ];

        this.legalProps = [
            'vel', 'dir', 'camera', 'ticks', 'isMenu', 'r', 'ref'
        ];
    }
    bump() {
        let player = this;

        if (player.ticks < 15) {
            player.dir *= -1;
        }

        player.vel.x *= 0.3;
        player.vel.y *= 0.3;

        player.vel.x += Math.sin(player.rot) * 12;
        player.vel.y -= Math.cos(player.rot) * 12;

        player.ticks = 0;
    }
    handleInt(target) {
        if (target.you == this.you) return;

        let dist = distF(this, target);

        let dp = (Math.sin(this.rot) * (this.pos.x - target.pos.x))
            - (Math.cos(this.rot) * (this.pos.y - target.pos.y));

        dp /= Math.sqrt(dist + 0.1);

        if (!(Math.sqrt(dist) < 128 && 1 / dp < -0.2)) return;

        let velness = Math.sqrt(this.vel.x ** 2 + this.vel.y ** 2);

        if (target.type == 'NPC' && velness > 0.2 && this.ticks >= 15) {
            this.isMenu = true;
            this.rot = 1.2;
        }

        if (target.type == 'Shooter') {
            this.health -= 1;
        }

        if (target.immortal) return;

        let oldHealth = target.health

        let dmg = Math.floor((dp - 0.001) * 10);

        target.health += Math.floor((dp - 0.001) * 10);

        target.vel.x += (target.pos.x - this.pos.x) * 0.1;
        target.vel.y += (target.pos.y - this.pos.y) * 0.1;

        if (target.health <= 0 && oldHealth > 0) {
            console.log(`Player ${target.you} died to a player ${this.you}`);
            this.health -= dmg;
            if (this.health > 100) this.health = 100;
            this.headCount += 500;
        }
    }
    handleTick(game) {
        let { entities, width, height } = game;

        this.headCount += 1 / 200;

        this.headCount = Math.round(this.headCount * 1000) / 1000;

        if (this.health <= 0) return;

        this.pos.x += this.vel.x;
        this.pos.y += this.vel.y;

        this.pos.x = Math.max(Math.min(this.pos.x, width / 2), - width / 2);
        this.pos.y = Math.max(Math.min(this.pos.y, height / 2), - height / 2);

        this.bounce(game);

        this.vel.x *= 0.9;
        this.vel.y *= 0.9;

        this.rot += 0.03 * this.dir;
        this.rot = this.rot % (Math.PI * 10);

        this.camera.x = -this.pos.x * 0.1 + this.camera.x * 0.9;
        this.camera.y = -this.pos.y * 0.1 + this.camera.y * 0.9;

        this.ticks++;

        for (let target of entities) {
            this.handleInt(target);
        }
    }
}

export default Player;